#!/usr/bin/perl
#
# Copyright (c) 2010-2015 Wanos Networks Pty(Ltd). All rights reserved.
#
# Modification of this software is not permitted under the licensing
# agreement. Please read the license as well as terms and conditions at
# http://wanos.org/license or email info@wanos.org for more information.
#
 
use Scalar::Util qw(looks_like_number);

my %sites = ();
my %peerips = ();
my %rows = ();
my %slots = ();
my %config = ();
my @nocfgsites;
my $max = 0;
my ($cpu) = $ARGV[0];
my $default_queue = $ARGV[1];

open(ROW, "<" , "/tce/etc/wanos/peersites") || die("Couldn't open the file. Error: $!\n");
while (<ROW>) 
{                    
	chomp;
	next if /^\s*\#/;           
	next unless /=/;            
	my ($key, $commastring) = split(/\s*=\s*/,$_,2);
	$commastring =~ s/(\$(\w+))/$rows{$2}/g;                             
	$rows{$key} = $commastring;                                      
	my @values = split(',',$commastring);
	$sites{$values[1]} = $values[0];
	if ($peerips{$values[1]} eq '') { $peerips{$values[1]} = $values[3]; }
}                                                                 
close ROW;

open(SLOT, "<" , "/tce/etc/wanos/siteslot") || warn("\n");
while (<SLOT>) 
{                    
	chomp;
	next if /^\s*\#/;           
	next unless /=/;            
	my ($key, $value) = split(/\s*=\s*/,$_,2);
	if ($value eq '') { $value = 0; }
	$slots{$key} = $value;
}                                                                 
close SLOT;

open(CONFIG, "<" , "/tce/etc/wanos/wanos.conf") || die("Couldn't open the file. Error: $!\n");
while (<CONFIG>)
{
  chomp;
  next if /^\s*\#/;
  next unless /=/;
  my ($key, $variable) = split(/\s*=\s*/,$_,2);
  $variable =~ s/(\$(\w+))/$config{$2}/g;
  $config{$key} = $variable;
}
close CONFIG;

my $rtt_delta = 2;
if ( $config{RSP_LEVEL} eq '2' ) { $rtt_delta = 1.6; }
elsif ( $config{RSP_LEVEL} eq '3' ) { $rtt_delta = 1.4; }
elsif ( $config{RSP_LEVEL} eq '4' ) { $rtt_delta = 1.2; }

my $ipno = qr/2(?:5[0-5]|[0-4]\d)|1\d\d|[1-9]?\d/x;

print <<EOF;
//
// Peer Sites
//
EOF

print "peer_route::SYS_Table -> [0]rsparp -> Strip(14) -> wanopout;\n";
print "rsparp[1] -> pq2;\n";

foreach (sort { $a <=> $b } keys(%sites) )
{ 
  print "peer_lan_tx_c" . $_ . "::Counter -> lantx_c;\n";
  print "peer_wan_tx_c" . $_ . "::Counter -> ";
  if (( $config{DEPLOYMENT_MODE} eq "tunnel") && ($peerips{$_}  !~ /^($ipno\.){3}$ipno$/ )) { print STDERR "Error : Site $_ did not have a valid tunnel peer ip. Site $_ traffic will be encapsulated, but not tunneled to a specific peer..\n"; }
  if (( $config{DEPLOYMENT_MODE} eq "tunnel") && ($peerips{$_}  =~ /^($ipno\.){3}$ipno$/ )) { print "WanosUDPIPEncap($config{IP}, 4050, $peerips{$_}, 4050) -> wanopout;\n"; }
  elsif (  $config{ENCAPPROTO} eq "udp") { print "WanosUDPIPEncap(SRC_ANNO, 4050, DST_ANNO, 4050) -> wanopout;\n"; }
  else { print "wanopout;\n"; }
}
print "\n";

foreach (sort { $a <=> $b } keys(%sites) )
{
 if ( $slots{$_} eq '' ) { $slots{$_} = 0; }
 print "dupq" . $_ . "::ThreadSafeQueue(" . ($default_queue * 2) . ") -> unq" . $_ . "::Unqueue ->  dedup" . $_ . " :: Dedup(ACTION 0, DS " . $_ . ", DS_START " . $slots{$_} . " , CLICK $config{CLICK}, LEVEL $config{DD_LEVEL}, DS_SIZE $config{DS_SIZE}, MEM_DENSE $config{MEM_DENSE} );\n";
 if ($_ gt $max) { $max = $_; }
}
print "\n";
@nocfgsites = (0 .. $max);

foreach (sort { $a <=> $b } keys(%sites) )
{
 my $peerip;
 my $wrtt = "false";
 if ( $peerips{$_} eq '') { $peerip = $config{IP}; }
 else { $peerip = $peerips{$_}; $wrtt = "true"; }

 my $compalgo = $config{COMPALGO};
 if ( $config{COMPMETHOD} eq 'stateless' ) { $compalgo = 1; }

 print "comp" . $_ . " :: WComp(SKIP_TRANSPORT_HEADER true, MIN_DATA_LEN 192, COMPRESSOR $compalgo, METHOD $config{COMPMETHOD});\n";
 print "wrtt_peer" . $_ . "::WRTT(LOCAL_IP $config{IP} , PEER_IP $peerip, DEFAULT_RTT 3000, RTT_HISTORY_COUNT 2, TIMER_INTERVAL 5, ENABLE $wrtt);\n";
 print "rsp_peer" . $_ . "::RSP(MODE $config{RSP_MODE}, LOCAL_IP $config{IP} , PEER_IP $peerip, WRTT wrtt_peer" . $_ . " , WCOMP comp" . $_ . " , RTT_DELTA " . $rtt_delta . ", MAX_RETRANSMIT_COUNT " . $config{RSP_MAX_RETRANSMIT_COUNT} . " , DEBUG $config{DEBUG}, ";
 if ( $config{RSP_DELAYED_ACK_MS} ne '') { print "DELAYED_ACK_MS $config{RSP_DELAYED_ACK_MS} );\n"; }
 else { print ");\n"; }
 print "dedup" . $_ . "[0]  -> compq" . $_ . "::ThreadSafeQueue(" . $default_queue . ") -> compunq" . $_ . "::Unqueue -> comp" . $_ . " -> rspq" . $_ . "::ThreadSafeQueue(" . $default_queue . ") -> rspunq" . $_ . "::Unqueue -> rsp_peer" . $_ . " -> peer_wan_tx_c" . $_ . ";\n";
 print "dedup" . $_ . "[1]  -> refq" . $_ . "::ThreadSafeQueue(100) -> refunq" . $_ . "::Unqueue -> IPMirror -> Unstrip(14) -> EtherMirror -> Strip(14) ->  peer_wan_tx_c" . $_ . ";\n";
 print "dedup" . $_ . "[2]  -> redupq" . $_ . "::ThreadSafeQueue(" . $default_queue . ") -> redupunq" . $_ . "::Unqueue -> peer_lan_tx_c" . $_ . ";\n";
 print "rsp_peer" . $_ . "[2] -> peer_route;\n";
 print "wrtt_peer" . $_ . "[1] -> peer_route;\n";
 print "wrtt_peer" . $_ . "[2] -> Unstrip(14) -> EtherMirror -> pq1;\n\n";
}
print "\n";
print "elementclass Wan_Out_Table {\n";
print "input -> out_lookup::RadixIPLookup(\n";

foreach (sort { $a <=> $b } keys(%rows) )  
{                                          
 my @values = split(',',$rows{$_});
 if (($values[0] ne '') && ($values[1] ne ''))
 {
 print "	" . $values[0] . " " . $values[1] . ",\n";
 }
}
print "	);\n";

foreach (sort { $a <=> $b } keys(%sites) )
{
 print "	out_lookup[" . $_ . "] -> [" . $_ . "]output;\n";
 $nocfgsites[$_] = 0;
}
foreach (@nocfgsites) { if ($_ != 0) {print "        out_lookup[" . $_ . "] -> [" . $_ . "]output;\n";} }
print "};\n\n";

print "elementclass Wan_In_Table {\n";
print "input -> in_lookup::WRadixIPLookup(\n";

foreach (sort { $a <=> $b } keys(%rows) )  
{                                          
 my @values = split(',',$rows{$_});
 if (($values[0] ne '') && ($values[1] ne ''))
 {
 print "	" . $values[0] . " " . $values[1] . ",\n";
 }
}
print "	);\n";

foreach (sort { $a <=> $b } keys(%sites) )
{
 print "	in_lookup[" . $_ . "] -> [" . $_ . "]output;\n";
 $nocfgsites[$_] = 0;
}
foreach (@nocfgsites) { if ($_ != 0) {print "        in_lookup[" . $_ . "] -> [" . $_ . "]output;\n";} }
print "};\n\n";

print "optable :: Wan_Out_Table;\n";
foreach (sort { $a <=> $b } keys(%sites) ) 
{
    if ($config{ENCAPPROTO} ne 'udp')
    {
	print "optable[" . $_ .  "] -> peer" . $_ . "::PeerProbe(MODE 1, INT 0, PROBE false) -> bypass" . $_ . "::Bypass -> peer_lan_rx_c" . $_ . "::Counter -> ";
	if ($config{MODE} eq 'Core') { print "dupq" . $_ . ";\n"; }
	else { print "compq" . $_ . ";\n"; }
 	print "peer". $_ . "[1] -> Paint(17) ->  checkout;\n";
 	print "bypass". $_ . "[1] ->  checkout;\n";
    }
    else
    {
	print "optable[" . $_ .  "] -> peer" . $_ . "::PeerProbe(MODE 1, INT 0, PROBE false) -> bypass" . $_ . "::Bypass -> peer_lan_rx_c" . $_ . "::Counter -> checkpaint" . $_ . "::CheckPaint(18) -> Paint(17) -> comp" . $_ . ";\n";
	if ($config{MODE} eq 'Core') { print "checkpaint" . $_ . "[1] -> dupq" . $_ . ";\n"; }
	else { print "checkpaint" . $_ . "[1] -> comp" . $_ . ";\n"; }
	print "peer". $_ . "[1] -> Paint(18) -> bypass" . $_ . ";\n";
	print "bypass" . $_ . "[1] -> udp_hello" . $_ . "::CheckPaint(18) -> Paint(17) -> udptee" . $_ . "::Tee -> checkout;\n";
	print "udp_hello" . $_ . "[1] -> checkout;\n";
 	if (( $config{DEPLOYMENT_MODE} eq "tunnel") && ($peerips{$_}  =~ /^($ipno\.){3}$ipno$/ ))
	{
		print "udptee" . $_ . "[1] -> RandomSample($config{UDPPEERRATE}) -> TruncateIPPayload -> WanosUDPIPEncap($config{IP}, 4050, $peerips{$_}, 4050) -> comp" . $_ . ";\n";
	}
	else
	{
 		print "udptee" . $_ . "[1] -> RandomSample($config{UDPPEERRATE}) -> TruncateIPPayload -> WanosUDPIPEncap(SRC_ANNO, 4050, DST_ANNO, 4050) -> comp" . $_ . ";\n";
	}
    }
}
foreach (@nocfgsites) { if ($_ != 0) {print "optable[" . $_ . "] -> Discard;\n";} }
print "\n";

foreach (sort { $a <=> $b } keys(%sites) ) 
{                                          
 print "handler" . $_ . ":: BypassHandler(BYPASS bypass" . $_ . ".active, PEERHELLO peer" . $_ . ".active, DD dedup" . $_ . ".ds_ready, HOLD $config{HOLD}, VERBOSE $config{VERBOSE});\n";
}
print "\n";

print "handlertable :: Wan_In_Table;\n";
foreach (sort { $a <=> $b } keys(%sites) )
 {
  print "handlertable[" . $_ . "] ->  handler" . $_ . ";\n"; 
 }
foreach (@nocfgsites) { if ($_ != 0) {print "handlertable[" . $_ . "] -> Discard;\n";} }
print "\n";

print "rtable :: Wan_In_Table;\n";
foreach (sort { $a <=> $b } keys(%sites) )
 {
  print "rtable[" . $_ . "] -> peer_wan_rx_c" . $_ . "::Counter -> wrtt_peer" . $_ . " -> [1]rsp_peer" . $_ . "[1] -> decomp" . $_ . "::WDeComp(SKIP_TRANSPORT_HEADER true) -> peer_lan_tx_c" . $_ . ";\n";
  print "decomp" . $_ . "[1] -> dupq" . $_ . ";\n";
  print "decomp" . $_ . "[2] -> handler" . $_ . ";\n";
 }
foreach (@nocfgsites) { if ($_ != 0) {print "rtable[" . $_ . "] -> Discard;\n";} }
print "\n\n";

my $i=0;
my $c=0;
if ($cpu > 1) { $i = 1; $c = 1;}
print "StaticThreadSched(";
foreach (sort { $a <=> $b } keys(%sites) )
{
print "unq" . $_ . " " . $i++ . ",";
if ($i >= $cpu) { $i=$c;}
print "compunq" . $_ . " " . $i++ . ",";
if ($i >= $cpu) { $i=$c;}
}
print ")\n\n";
